{{
/**
 * This object presents a heap. The heap is a large contiguous byte array.
 * The object provides all the methods for allocating and freeing blocks of memory
 * and does all the housekeeping to preserve the heap integrity.
 * Allocation of blocks is done using a first-fit scheme. 
 *
 * Example:
 * OBJ
 *   hp1: "heap"
 *   hp2: "heap"
 * VAR
 *   byte heapArray[1024]           'heapArray will be used to hold a heap
 *   word block
 * PUB
 *   hp1.create(@heapArray,1024)    'create heap with size 1024 bytes
 *   if (hp1.available => 120)
 *     block := hp1.allocate(120)   'allocate a block of 120 bytes
 *     'do something with block
 *     hp1.free(block)
 *   else
 *     'error No block allocated
 *
 * It is possible to create a heap inside a heap.
 * Example:
 *   hp1.create(@heapArray,1024)
 *   block := hp1.allocate(120)           'allocate a block
 *   hp2.create(block,hp1.length(block)) 'convert block to heap
 * This is useful if a task must keep track of multiple blocks (from heap hp2) but the task may be aborted.
 * In that case a single call to free will free all blocks allocated from heap hp2.
 *   hp1.free(hp2.heap)
 * 
 * Revision History:
 * Jan 2, 2008 v1.1: Initial release.
 * @author Peter Verkaik (verkaik6@zonnet.nl)
 */

/*
  Implementation of the heap
  heapStart:   DW   NEXT1        //pointer to next block (b15 set if this block allocated)
               DW   SIZE1        //number of bytes in this block
  <blkaddr>    DS   SIZE1        //blkaddr is returned block address
  NEXT1:       DW   NEXT2
               DW   SIZE2
  <blkaddr>    DS   SIZE2
               ..
               DW   heapEnd
               DW   SIZEn
  <blkaddr>    DS   SIZEn
  heapEnd:     DW   0            //closing 0 word

  Minimal heapsize is 7 bytes (word NEXT1, word SIZE1,allocatable byte, word 0)
*/
}}


PUB create(base,size)
  '/**
  ' * Create a heap.
  ' * This transforms a byte array into a heap.
  ' *
  ' * @param base Start address of heap
  ' * @param size Size of heap in bytes (size => 7 bytes)
  ' */

PUB heap: base
  '/**
  ' * Get heap address.
  ' * This returns the heap base address as passed to method create.
  ' *
  ' * @return Heap base address
  ' */

PUB integrity: YesNo | prev,nxt,temp,size
  '/**
  ' * Test integrity of heap.
  ' * Checks all blocks (allocated and free).
  ' *
  ' * @return False if heap integrity lost.
  ' */

PUB available: value | prev,nxt,len,maximum,temp
  '/**
  ' * Get largest available size for free block.
  ' *
  ' * @return Number of bytes in largest free block or 0.
  ' */

PUB allocate(len): value | prev,nxt,temp,size
  '/**
  ' * Allocate a block.
  ' *
  ' * @param len Block size in bytes (>0)
  ' * @return Block address if block allocated or 0.
  ' */

PUB free(addr) | prev,nxt,temp,temp2
  '/**
  ' * Free an allocated block.
  ' * This returns memory to the heap. The block must not be accessed afterwards.
  ' *
  ' * @param addr Address of previously allocated block.
  ' */

PUB length(addr): value  
  '/**
  ' * Get length of allocated block.
  ' *
  ' * @param addr Address of allocated block.
  ' * @return Length in bytes or 0.
  ' */
  